#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <mpi.h>
#include "particulas.h"
#include "vectores.h"

#define SNAPSHOTS 5

int main(int argc, char* argv[])
{
  size_t i,n_objs,n_iter, n_reps;
  char* endp;
  particula_t* particulas;
  particula_t* repulsores;

  int rank, size;
  size_t chunk, min, max;

  MPI_Init(&argc,&argv);
  MPI_Comm_rank(MPI_COMM_WORLD,&rank);
  MPI_Comm_size(MPI_COMM_WORLD,&size);

  n_objs=(size_t)strtol(argv[1],&endp,10);

  if((n_objs%(10*size))!=0)
    n_objs=(1+(n_objs/(10*size)))*10*size;

  n_iter=(size_t)strtol(argv[2],&endp,10);
  n_reps=(size_t)(n_objs/10);

  if(rank==0) /* Master */
    {
      particulas=inicializar(n_objs);
      repulsores=inicializar(n_reps);
      fprintf(stdout,"Numero efectivo de particulas=%d / repulsores=%d\n",n_objs, n_reps);
      fprintf(stderr,"Iteracion: _____");
    }
  else
    {
      particulas=(particula_t*)malloc(sizeof(particula_t)*n_objs);
      bzero((void*)particulas,sizeof(particula_t)*n_objs);

      repulsores=(particula_t*)malloc(sizeof(particula_t)*n_reps);
      bzero((void*)repulsores,sizeof(particula_t)*n_reps);
    }

  for(i=0;i<n_iter;i++)
    {
      MPI_Bcast(particulas, 4*n_objs, MPI_DOUBLE, 0, MPI_COMM_WORLD);
      MPI_Bcast(repulsores, 4*n_reps, MPI_DOUBLE, 0, MPI_COMM_WORLD);
      
      chunk = n_objs / size;
      min   = chunk  * rank;
      max   = min    + chunk;
      actualizar_particulas(particulas,n_objs,min,max,repulsores,n_reps);      

      MPI_Gather(&(particulas[min]),4*chunk, MPI_DOUBLE, particulas, 4*chunk, MPI_DOUBLE, 0, MPI_COMM_WORLD);

      chunk = n_reps / size;
      min   = chunk  * rank;
      max   = min    + chunk;
      actualizar_repulsores(repulsores,n_reps,min,max);      

      MPI_Gather(&(repulsores[min]), 4*chunk, MPI_DOUBLE, repulsores, 4*chunk, MPI_DOUBLE, 0, MPI_COMM_WORLD);
      if(rank==0)
	{
	  fprintf(stderr,"\b\b\b\b\b%5d",i);
	
#ifdef DEBUG
	  if(i%(n_iter/SNAPSHOTS)==0)
	    {
	      fprintf(stdout,"PARTICULAS %d\n",i);
	      imprimir(particulas,n_objs);
	    }
#endif
	}
    }

  if(rank==0)
    {
      fprintf(stdout,"PARTICULAS\n");
      imprimir(particulas,n_objs);
      fprintf(stdout,"REPULSORES\n");
      imprimir(repulsores,n_reps);
    }

  return 0;
}

