#ifndef _PARTICULAS_H_
#define _PARTICULAS_H_

#include <stdlib.h>
#include <stdio.h>
#include "vectores.h"

#define RADIO 5.0
#define MAX_VELOCIDAD 7.0
#define DISPERSION 0.5
#define MAX_FUERZA 9.0
#define REBOTE 5.0
#define REPULSION_DIST 5.0
#define TIEMPO  1.0/10.0

#define MASA_CRITICA   50
#define CONST_ENTROPIA 0.0112

#define FACTOR_SEP 0.03
#define FACTOR_COH 0.45
#define FACTOR_DIS 0.02
#define FACTOR_REP 0.25
#define FACTOR_ENT 0.05
#define FACTOR_ROT 0.20

#define MIN_X -50.0
#define MAX_X  50.0
#define MIN_Y -50.0
#define MAX_Y  50.0

//#define DEBUG 1

typedef struct particula_st
{
  vector_t posicion;
  vector_t velocidad;
} particula_t;

void buscar(particula_t* o, vector_t* pos, vector_t* fuerza);
int  vecino(particula_t* o1, particula_t* o2, double radio);
int  tiene_vecinos(particula_t* o, particula_t* vo, size_t n_objs, double radio);
void separacion(particula_t* o, particula_t* vo, size_t n_objs, double radio, vector_t* fuerza);
void entropia(particula_t* o, particula_t* vo, size_t n_objs, double radio, vector_t* fuerza);
void cohesion(particula_t* o, particula_t* vo, size_t n_objs, double radio, vector_t* fuerza);
void dispersion(vector_t* fuerza, double rango);
void rotacion(particula_t* o, vector_t* fuerza);
void repulsion(particula_t* o, particula_t* vr, size_t n_reps, vector_t* fuerza);
void combinar(particula_t* o, particula_t* vo, size_t n_objs, particula_t* vr, size_t n_reps, vector_t* fuerza);
void acelerar(particula_t* o, vector_t* fuerza);
void mover(particula_t* o);

void actualizar_repulsores(particula_t* vr, size_t n_reps, size_t min_rep, size_t max_rep);
void actualizar_particulas(particula_t* vo, size_t n_objs, size_t min_obj, size_t max_obj, particula_t* vr, size_t n_reps);

void nueva_particula(particula_t* o);
particula_t* inicializar(size_t n_objs);

void imprimir_particula(particula_t* o);
void imprimir(particula_t* vo, size_t n_objs);

#endif
